<?php
/**
 * 2007-2015 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    vnphpexpert <vnphpexpert@gmail.com>
 *  @copyright 2007-2015 vnphpexpert
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of vnphpexpert
 */

/**
 * @since 1.5.0
 */
if (!class_exists('Securepaylib')) {
    require_once(_PS_MODULE_DIR_.'/secureapi/controllers/library/class-secure-api.php');
}
class SecureApiValidationModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        $paymentToken = Tools::getValue('token');
        $order_id = Tools::getValue('order_id'). time();
        $token_saved = Tools::getValue('token_saved');
        $client_id = Configuration::get('SECURE_API_CLIENT_ID');
        $client_secret = Configuration::get('SECURE_API_CLIENT_SECRET');
        $client_merchant_code = Configuration::get('SECURE_API_MERCHANT_CODE');
        $test_mode = Configuration::get('SECURE_API_TEST_MODE');
        $transaction_type = Configuration::get('SECURE_API_TRANSACTION_TYPE');
        $payment_status = Configuration::get('SECURE_API_REFUND_STATUS');
        $store_id = Configuration::get('SECURE_API_STORE_ID');
        $customer_id = null;
        $cart = $this->context->cart;
        $customer = new Customer($cart->id_customer);
        if ($customer->id) {
            $customer_id = $customer->id;
        }

        $params = [
            'client_id'=>$client_id,
            'client_secret'=>$client_secret,
            'merchant_code'=>$client_merchant_code,
            'sandbox_mode'=>$test_mode,
            'transaction_type'=>$transaction_type,
            'customer_id'=>$customer_id,
            'order_id'=>$order_id,
            'store_id'=>$store_id
            ];
        $securepay_api = new Securepaylib($params);
        $amount = number_format($cart->getOrderTotal(true, Cart::BOTH), 2, '.', '') * 100;
        $payment = $securepay_api->createPayment($paymentToken, $amount);
        if (isset($payment['status']) && isset($payment['gatewayResponseCode']) && $payment['status'] == 'paid' && in_array($payment['gatewayResponseCode'], array('00'))) {
            $transaction_id = $payment['bankTransactionId'];
            //save card
            if ($token_saved ==1) {
                $payment_instrument = $securepay_api->createPaymentInstrument($customer_id, $paymentToken);
                if (empty($payment_instrument['error']) && $token_saved == 1) {
                    $card_token = $payment_instrument['token'];
                    $customerCode = $payment_instrument['customerCode'];
                    //check exist token

                    $check_token = 'SELECT * FROM `'._DB_PREFIX_.'secure_api_saved_card`
                            WHERE card_token = "'.pSQL($card_token).'"
                            AND customer_code = "'.pSQL($customerCode).'"
                            AND user_id = '.(int)$customer_id;

                    $exist = Db::getInstance()->getRow($check_token);
                    if (!$exist) {
                        $brandType = $payment_instrument['brandType'];
                        $brandCategory = $payment_instrument['brandCategory'];
                        $scheme = $payment_instrument['scheme'];
                        $last4 = $payment_instrument['last4'];
                        $expiryMonth = $payment_instrument['expiryMonth'];
                        $expiryYear = $payment_instrument['expiryYear'];
                        $createdAt = $payment_instrument['createdAt'];
                        Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'secure_api_saved_card` (`user_id`,`card_token`,`customer_code`,`brand_type`,`brand_category`,`scheme`,`last4`,`expiry_month`,`expiry_year`,`created_time`)
										VALUES ( '.(int)$customer_id.',
										 \''.pSQL(trim($card_token)).'\', 
										 \''.pSQL(trim($customerCode)).'\', 
										 \''.pSQL(trim($brandType)).'\', 
										 \''.pSQL(trim($brandCategory)).'\', 
										 \''.pSQL(trim($scheme)).'\', 
										 \''.pSQL(trim($last4)).'\', 
										 \''.pSQL(trim($expiryMonth)).'\', 
										 \''.pSQL(trim($expiryYear)).'\', 
										 \''.pSQL(trim($createdAt)).'\'
										 )');
                    }
                }
            }

            $currency = new Currency((int) $cart->id_currency);
            //success transaction
            $name = $this->module->displayName;
            $tran_id = array('transaction_id' =>$transaction_id);
            $c_id = (int) $currency->id;
            $am_per = $amount / 100;
            $this->module->validateOrder($cart->id, $payment_status, $am_per, $name, null, $tran_id, $c_id, false, $this->context->cart->secure_key);

            //save to order status
            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'secure_api_order_status` (`cart_id`,`order_id`,`secure_order_type`,`created_time`,`updated_time`)
										VALUES ( '.(int)$this->module->currentOrder.',
										 \''.pSQL(trim($order_id)).'\', 
										 \''.pSQL(trim($transaction_type)).'\', 
										  NOW(),
										  NOW()
										 )');
            Tools::redirect('index.php?controller=order-confirmation&id_cart='.(int)$cart->id.'&id_module='.(int)$this->module->id.'&id_order='.$this->module->currentOrder.'&key='. $customer->secure_key);
        } else {
            $returnMessage = 'Your transaction was unsuccessful, please check your details and try again. Error detail: ' . $payment['errorCode'] . ' - ' .  $payment['errorDetail'];
            $this->errors[] = $returnMessage;
            $this->redirectWithNotifications($this->context->link->getPageLink('order', true, null, array(
                'step' => '3')));
        }
    }
}
