<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Software License Agreement
 * that is bundled with this package in the file LICENSE.txt.
 *
 * @author    SecurePay - https://www.securepay.com.au/
 * @copyright PrestaShop SA
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class Securepaylib
{
    public $log;
    public $_checkout_sandbox_payment_url = 'https://payments-stest.npe.auspost.zone/v2/';
    public $_checkout_live_payment_url = 'https://payments.npe.auspost.zone/v2/';

    public $_sandbox_url_js = 'https://payments-stest.npe.auspost.zone/v3/ui/client/securepay-ui.min.js';
    public $_live_url_js = 'https://payments.auspost.net.au/v3/ui/client/securepay-ui.min.js';

    public $_sandbox_token_url = 'https://welcome.api2.sandbox.auspost.com.au/oauth/token';
    public $_live_token_url = 'https://welcome.api2.auspost.com.au/oauth/token';

    public $client_id;
    public $client_secrect;
    public $merchantCode;
    public $sandbox_mode;
    public $transaction_type;
    public $customer_id;
    public $order_id;
    public $bearerToken = '';
    public $store_id = '';
    public $stored_customer = false;

    const SCOPE_PAYMENT_READ = 'https://api.payments.auspost.com.au/payhive/payments/read';
    const SCOPE_PAYMENT_WRITE = 'https://api.payments.auspost.com.au/payhive/payments/write';
    const SCOPE_PAYMENT_INSTRUMENT_READ = 'https://api.payments.auspost.com.au/payhive/payment-instruments/read';
    const SCOPE_PAYMENT_INSTRUMENTS_WRITE = 'https://api.payments.auspost.com.au/payhive/payment-instruments/write';

    const ERROR_CURL = 'curl_error';
    const ERROR_BEARER_TOKEN_NOT_SET = 'bearer_token_not_set';
    const ERROR_MERCHANT_CODE_NOT_SET = 'merchant_code_not_set';
    const ERROR_INVALID_RESPONSE = 'invalid_server_response';


    public function __construct($params)
    {
        $this->client_id = $params['client_id'];
        $this->client_secrect = $params['client_secret'];
        $this->merchantCode = $params['merchant_code'];
        $this->sandbox_mode = $params['sandbox_mode'];
//        $this->transaction_type = $params['sandbox_mode'];
        $this->transaction_type = $params['transaction_type'];
        $this->customer_id = $params['customer_id'];
        $this->order_id = $params['order_id'];
        $this->store_id = $params['store_id'];
        //set $clientSecret
        $get_token = $this->initToken();
        if ($get_token['error']) {
            //show error
        } else {
            $this->bearerToken = $get_token['token'];
        }
    }

    public function getbearerToken()
    {
        return $this->bearerToken;
    }

    public function getUrlJs()
    {
        if ($this->sandbox_mode == 1) {
            return $this->_sandbox_url_js;
        } else {
            return $this->_live_url_js;
        }
    }

    public function getTokenUrl()
    {
        if ($this->sandbox_mode == 1) {
            return $this->_sandbox_token_url;
        } else {
            return $this->_live_token_url;
        }
    }

    public function getBasePaymentUrl()
    {
        if ($this->sandbox_mode == 1) {
            return  $this->_checkout_sandbox_payment_url;
        } else {
            return $this->_checkout_live_payment_url;
        }
    }

    public function getPaymentUrl()
    {
        if ($this->sandbox_mode == 1) {
            $checkout_payment_url =  $this->_checkout_sandbox_payment_url;
        } else {
            $checkout_payment_url = $this->_checkout_live_payment_url;
        }
        switch ($this->transaction_type) {
            case 'PAYMENT':
                $payment_url =  $checkout_payment_url .'payments';
                if ($this->stored_customer) {
                    $payment_url = $checkout_payment_url .'customers/'.$this->customer_id .'/payments';
                }
                break;
            case 'PREAUTH':
                $payment_url = $checkout_payment_url .'payments/preauths';
                break;
        }
        return $payment_url;
    }

    public function initToken()
    {
        $authUrl  = $this->getTokenUrl();
        $clientId = $this->client_id;
        $clientSecret = $this->client_secrect;

		$audience = 'https://api.payments.auspost.com.au';

        $ch = curl_init();
        $scope = array(self::SCOPE_PAYMENT_READ, self::SCOPE_PAYMENT_WRITE, self::SCOPE_PAYMENT_INSTRUMENT_READ, self::SCOPE_PAYMENT_INSTRUMENTS_WRITE);
        curl_setopt($ch, CURLOPT_URL, $authUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, 'grant_type=client_credentials&audience=' . rawurlencode( $audience ));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_USERPWD, $clientId . ":" . $clientSecret);

        $headers = array();
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);

        $resultArray = json_decode($result, true);
        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['error'])) {
            return ['error' => true,
                'errorCode' => $resultArray['error'],
                'errorDetail' => $resultArray['error_description']];
        }
        return ['error' => false, 'token'=> $resultArray['access_token']];
    }



    public function log($filelog, $contentlog)
    {
        file_put_contents(dirname(dirname(dirname(__FILE__))) . '/' . $filelog, $contentlog, FILE_APPEND);
    }

    public function getmerchantCode()
    {
        return $this->merchantCode;
    }

    /**
     * Create payment
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-create-payment
     * @param string $paymentToken
     * @param int $amount
     * @return array
     */
    public function createPayment($paymentToken, $amount)
    {
        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }

        $ch = curl_init();
        $checkout_payment_url = $this->getPaymentUrl();
        curl_setopt($ch, CURLOPT_URL, $checkout_payment_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $postFields = [
            'amount' => (int) $amount,
            'merchantCode' => $this->merchantCode,
            'token' => $paymentToken,
            'ip' => $this->getTheUserIp(),
            'orderId' =>$this->storeUniquify($this->order_id),
            'metadata' => $this->getMetadata(),
        ];
        $this->log('fgct.txt', $postFields);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postFields));
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        /*if (isset($idempotencyKey)) {
            $headers[] = 'Idempotency-Key: ' . $idempotencyKey;
        }*/
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);
        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }
        if (!isset($resultArray['status'])) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => $resultArray
            ];
        }
        if ($resultArray['status'] !== 'paid') {
            return ['error' => true,
                'errorCode' => 'payment_status_' . $resultArray['status'],
                'errorDetail' => $resultArray['gatewayResponseCode'] . ' - ' . $resultArray['gatewayResponseMessage']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }

    /**
     * Create Payment (Stored Customer)
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-create-payment-stored-customer
     * @param string $paymentToken
     * @param int $amount
     * @return array
     */

    public function createPaymentStoredCustomer($amount, $paymentToken)
    {
        $this->stored_customer = true;
        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }

        $ch = curl_init();
        $checkout_payment_url = $this->getPaymentUrl();
        curl_setopt($ch, CURLOPT_URL, $checkout_payment_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $postFields = [
            'amount' => (int) $amount,
            'merchantCode' => $this->merchantCode,
            'token' => $paymentToken,
            'ip' => $this->getTheUserIp(),
            'orderId' =>$this->storeUniquify($this->order_id),
            'metadata' => $this->getMetadata(),
        ];
        $this->log('fgct.txt', $postFields);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postFields));
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        /*if (isset($idempotencyKey)) {
            $headers[] = 'Idempotency-Key: ' . $idempotencyKey;
        }*/
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);

        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }
        if (!isset($resultArray['status'])) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => $resultArray
            ];
        }
        if ($resultArray['status'] !== 'paid') {
            return ['error' => true,
                'errorCode' => 'payment_status_' . $resultArray['status'],
                'errorDetail' => $resultArray['gatewayResponseCode'] . ' - ' . $resultArray['gatewayResponseMessage']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }


    /**
     * Create payment instrument for specified user account
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-create-payment-instrument
     * @param string $customerCode
     * @param string $paymentToken
     * @param string $ipAddress
     * @return array
     */
    public function createPaymentInstrument($customerCode, $paymentToken)
    {
        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->getBasePaymentUrl() . 'customers/' . $this->storeUniquify($customerCode) . '/payment-instruments/token');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        $headers[] = 'token: ' . $paymentToken;
        $headers[] = 'ip: ' . $this->getTheUserIp();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);
        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => self::ERROR_INVALID_RESPONSE_DETAILS];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }

    /**
     * capture PreAuth Payment
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-create-capture-payment
     * @return array
     */
    public function capturePreAuthPayment($amount)
    {
        $this->stored_customer = true;

        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->getBasePaymentUrl() . 'payments/preauths/' . $this->storeUniquify($this->order_id) . '/capture');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $postFields = [
            'amount' => (int) $amount,
            'merchantCode' => $this->merchantCode,
            'ip' => $this->getTheUserIp(),
            'metadata' => $this->getMetadata(),
        ];

        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postFields));
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        /*if (isset($idempotencyKey)) {
            $headers[] = 'Idempotency-Key: ' . $idempotencyKey;
        }*/
        $headers[] = 'Idempotency-Key: ' . $this->storeUniquify($this->order_id);
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);

        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }
        if (!isset($resultArray['status'])) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => $resultArray
            ];
        }
        if ($resultArray['status'] !== 'paid') {
            return ['error' => true,
                'errorCode' => 'payment_status_' . $resultArray['status'],
                'errorDetail' => $resultArray['gatewayResponseCode'] . ' - ' . $resultArray['gatewayResponseMessage']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }

    /**
     * Cancel PreAuth Payment
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-cancel-preauth-payment
     * @return array
     */
    public function cancelPreAuthPayment()
    {
        $this->stored_customer = true;

        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->getBasePaymentUrl() . 'payments/preauths/' . $this->storeUniquify($this->order_id) . '/cancel');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $postFields = [
            'merchantCode' => $this->merchantCode,
            'ip' => $this->getTheUserIp(),
            'metadata' => $this->getMetadata(),
        ];

        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postFields));
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        /*if (isset($idempotencyKey)) {
            $headers[] = 'Idempotency-Key: ' . $idempotencyKey;
        }*/
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);

        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }
        if (!isset($resultArray['status'])) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => $resultArray
            ];
        }
        if ($resultArray['status'] !== 'paid') {
            return ['error' => true,
                'errorCode' => 'payment_status_' . $resultArray['status'],
                'errorDetail' => $resultArray['gatewayResponseCode'] . ' - ' . $resultArray['gatewayResponseMessage']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }


    /**
     * Refund Payment
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-refund-payment
     * @param int $amount
     * @return array
     */

    public function refundPayment($amount)
    {
        $this->stored_customer = true;

        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->getBasePaymentUrl() . 'orders/' . $this->storeUniquify($this->order_id) . '/refunds');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $postFields = [
            'amount' => (int) $amount,
            'merchantCode' => $this->merchantCode,
            'ip' => $this->getTheUserIp(),
            'metadata' => $this->getMetadata(),
        ];

        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postFields));
        curl_setopt($ch, CURLOPT_POST, 1);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
       /* if (isset($idempotencyKey)) {
            $headers[] = 'Idempotency-Key: ' . $idempotencyKey;
        }*/
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);

        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => 'Invalid response from server'];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }
        if (!isset($resultArray['status'])) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => $resultArray
            ];
        }
        if ($resultArray['status'] !== 'paid') {
            return ['error' => true,
                'errorCode' => 'payment_status_' . $resultArray['status'],
                'errorDetail' => $resultArray['gatewayResponseCode'] . ' - ' . $resultArray['gatewayResponseMessage']];
        }
        $resultArray['error'] = false;
        return $resultArray;
    }

    /**
     * Create payment instrument for specified user account
     * https://auspost.com.au/payments/docs/securepay/#card-payments-rest-api-create-payment-instrument
     * @param string $customerCode
     * @param string $paymentToken
     * @param string $ipAddress
     * @return array
     */
    public function listPaymentInstrument($customerCode)
    {
        if (!isset($this->merchantCode)) {
            return ['error' => true,
                'errorCode' => self::ERROR_MERCHANT_CODE_NOT_SET,
                'errorDetail' => 'Merchant code is not set. Have you called init function?'];
        } elseif (!isset($this->bearerToken)) {
            return ['error' => true,
                'errorCode' => self::ERROR_BEARER_TOKEN_NOT_SET,
                'errorDetail' => 'Bearer token is not set. Have you called init function?'];
        }
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->getBasePaymentUrl() . 'customers/' . $this->storeUniquify($customerCode) . '/payment-instruments');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Authorization: Bearer ' . $this->bearerToken;
        $headers[] = 'ip: ' . $this->getTheUserIp();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            return ['error' => true,
                'errorCode' => self::ERROR_CURL,
                'errorDetail' => curl_error($ch)];
        }
        curl_close($ch);
        $resultArray = json_decode($result, true);
        if ($resultArray === null) {
            return ['error' => true,
                'errorCode' => self::ERROR_INVALID_RESPONSE,
                'errorDetail' => self::ERROR_INVALID_RESPONSE_DETAILS];
        }
        if (isset($resultArray['errors'])) {
            return ['error' => true,
                'errorCode' => $resultArray['errors'][0]['code'],
                'errorDetail' => $resultArray['errors'][0]['detail']];
        }

        $resultArray['error'] = false;
        return $resultArray;
    }


    private function storeUniquify($string)
    {
        if ($this->isRawString($string)) {
            return $this->store_id . '_' . $string;
        }
        return $string;
    }

    private function isRawString($string)
    {
        return Tools::substr($string, 0, Tools::strlen($this->store_id)) !== $this->store_id;
    }

    public function getTheUserIp()
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip;
    }
    public function getMetadata()
    {
        $version = _PS_VERSION_;
        $version_array = explode('.', _PS_VERSION_);
        // if(count($version_array) > 3){
        //     $version = $version_array[0] . '.' . $version_array[1] . '.' . $version_array[2];
        // }
        
        return [
            'shoppingCartPlatform'=>[
                'version' => $version,
                'name' => 'PrestaShop'
                ]
            ];
            
    }
}
